package com.ruoyi.web.yh.report.service.impl;

import java.util.List;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.web.yh.report.domain.bo.ReportBO;
import com.ruoyi.web.yh.reportProd.service.ReportProdService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.web.yh.report.mapper.ReportMapper;
import com.ruoyi.web.yh.report.domain.Report;
import com.ruoyi.web.yh.report.domain.ReportVO;
import com.ruoyi.web.yh.report.domain.ReportQuery;
import com.ruoyi.web.yh.report.service.ReportService;
import org.springframework.transaction.support.TransactionTemplate;

/**
 * 报表Service业务层处理
 *
 * @author ruoyi
 * @date 2024-10-18
 */
@Service
public class ReportServiceImpl implements ReportService
{
    @Autowired
    private ReportMapper reportMapper;

    @Autowired
    private TransactionTemplate transactionTemplate;

    @Autowired
    private ReportProdService reportProdService;

    /**
     * 查询报表
     *
     * @param reportId 报表主键
     * @return 报表
     */
    @Override
    public ReportVO selectReportByReportId(Long reportId)
    {
        return reportMapper.selectReportByReportId(reportId);
    }

    /**
     * 查询报表列表
     *
     * @param report 报表
     * @return 报表
     */
    @Override
    public List<ReportVO> selectReportList(ReportQuery report)
    {
        return reportMapper.selectReportList(report);
    }

    /**
     * 新增报表
     *
     * @param report 报表
     * @return 结果
     */
    @Override
    public int insertReport(Report report)
    {
        report.setCreateTime(DateUtils.getNowDate());
        return reportMapper.insertReport(report);
    }

    /**
     * 修改报表
     *
     * @param report 报表
     * @return 结果
     */
    @Override
    public int updateReport(Report report)
    {
        report.setUpdateTime(DateUtils.getNowDate());
        return reportMapper.updateReport(report);
    }

    /**
     * 批量删除报表
     *
     * @param reportIds 需要删除的报表主键
     * @return 结果
     */
    @Override
    public int deleteReportByReportIds(Long[] reportIds)
    {
        return reportMapper.deleteReportByReportIds(reportIds);
    }

    /**
     * 删除报表信息
     *
     * @param reportId 报表主键
     * @return 结果
     */
    @Override
    public int deleteReportByReportId(Long reportId)
    {
        return reportMapper.deleteReportByReportId(reportId);
    }

    @Override
    public int addReport(ReportBO bo) {
        ServiceUtil.assertion(bo == null, "参数错误");

        Integer result = transactionTemplate.execute(status -> {
            // todo 新增主表
            int insert = this.insertReport(bo);
            ServiceUtil.assertion(insert != 1, "新增报表失败");

            // TODO 新增子表
            int batchInsert = reportProdService.batchInsertByReportId(bo.getProductList(), bo.getReportId());
            ServiceUtil.assertion(batchInsert != bo.getProductList().size(), "新增报表产量明细失败");

            return insert;
        });

        return result == null ? 0 : result;
    }
}
